@@ -74,8 +74,8 @@ class Access:
                 client_id=client_id, client_secret=client_secret,
             )
 
-        self.token = Token.astoken(token, refresh_buffer) if token else None
-        self.refresh_token = Token.astoken(refresh_token) if refresh_token else None
+        self.token = Token.astoken(token, refresh_buffer) if token is not None else None
+        self.refresh_token = Token.astoken(refresh_token) if refresh_token is not None else None
 
         login = token is None if login is None else login
         if login and not self.refresh_token and self.username and self.password:
